@@ -1,13 +1,12 @@
 /*
  * kbiff.cpp
- * Copyright (C) 1999-2002 Kurt Granroth <granroth@kde.org>
+ * Copyright (C) 1999-2008 Kurt Granroth <granroth@kde.org>
  *
  * This file contains the implementation of the main KBiff
  * widget
- *
- * $Id: kbiff.cpp,v 1.10 2002/11/02 03:36:35 gran Exp $
  */
 #include "kbiff.h"
+#include "kbiff.moc"
 #include <qmovie.h>
 #include <qtooltip.h>
 
@@ -25,17 +24,19 @@
 #include "setupdlg.h"
 #include "notify.h"
 #include "status.h"
+#include "led.h"
 
 #include <unistd.h>
 
 #include <dcopclient.h>
 
-KBiff::KBiff(DCOPClient *client, QWidget *parent)
-    : QLabel(parent),
-      DCOPObjectProxy(client),
+KBiff::KBiff(DCOPClient *client_, QWidget *parent_)
+    : DCOPObjectProxy(client_),
+      QLabel(parent_),
       statusTimer(0),
       status(0),
-      statusChanged(true)
+      statusChanged(true),
+      mled( new Led("mled") )
 {
     setBackgroundMode(X11ParentRelative);
 
@@ -52,7 +53,7 @@ KBiff::KBiff(DCOPClient *client, QWidget *parent)
     statusList.setAutoDelete(true);
 
     // register with DCOP
-    registerMe(client);
+    registerMe(client_);
 
     reset();
 }
@@ -62,6 +63,7 @@ KBiff::~KBiff()
     monitorList.clear();
     notifyList.clear();
     statusList.clear();
+    delete mled;
 
     // we no longer want to be registered
     DCOPClient *client = kapp->dcopClient();
@@ -76,27 +78,30 @@ KBiff::~KBiff()
     client->detach();
 }
 
-void KBiff::processSetup(const KBiffSetup* setup, bool run)
+void KBiff::processSetup(const KBiffSetup* setup_, bool run_)
 {
     // General settings
-    isSecure    = setup->getSecure();
-    profile     = setup->getProfile();
-    mailClient  = setup->getMailClient();
-    sessions    = setup->getSessionManagement();
-    skipcheck   = setup->getCheckStartup();
-    noMailIcon  = setup->getNoMailIcon();
-    newMailIcon = setup->getNewMailIcon();
-    oldMailIcon = setup->getOldMailIcon();
-    noConnIcon  = setup->getNoConnIcon();
+    isSecure    = setup_->getSecure();
+    profile     = setup_->getProfile();
+    mailClient  = setup_->getMailClient();
+    sessions    = setup_->getSessionManagement();
+    skipcheck   = setup_->getCheckStartup();
+    noMailIcon  = setup_->getNoMailIcon();
+    newMailIcon = setup_->getNewMailIcon();
+    oldMailIcon = setup_->getOldMailIcon();
+    noConnIcon  = setup_->getNoConnIcon();
+    stoppedIcon = setup_->getStoppedIcon();
 
     // New mail
-    systemBeep     = setup->getSystemBeep();
-    runCommand     = setup->getRunCommand();
-    runCommandPath = setup->getRunCommandPath();
-    playSound      = setup->getPlaySound();
-    playSoundPath  = setup->getPlaySoundPath();
-    notify         = setup->getNotify();
-    dostatus       = setup->getStatus();
+    systemBeep     = setup_->getSystemBeep();
+    runCommand     = setup_->getRunCommand();
+    runCommandPath = setup_->getRunCommandPath();
+    runResetCommand     = setup_->getRunResetCommand();
+    runResetCommandPath = setup_->getRunResetCommandPath();
+    playSound      = setup_->getPlaySound();
+    playSoundPath  = setup_->getPlaySoundPath();
+    notify         = setup_->getNotify();
+    dostatus       = setup_->getStatus();
 
     // if we aren't going the status route, we should at least
     // provide a tooltip!
@@ -106,13 +111,13 @@ void KBiff::processSetup(const KBiffSetup* setup, bool run)
         QToolTip::remove(this);
 
     // set all the new mailboxes
-    setMailboxList(setup->getMailboxList(), setup->getPoll());
+    setMailboxList(setup_->getMailboxList(), setup_->getPoll());
 
     // change the dock state if necessary
-    if (docked != setup->getDock())
+    if (docked != setup_->getDock())
         dock();
 
-    if (run && !skipcheck)
+    if (run_ && !skipcheck)
         start();
     skipcheck = false;
 
@@ -144,7 +149,7 @@ void KBiff::processSetup(const KBiffSetup* setup, bool run)
       status = new KBiffStatus(this, profile, statusList);
     }
 
-    delete setup;
+    delete setup_;
 }
 
 void KBiff::setMailboxList(const QList<KBiffMailbox>& mailbox_list, unsigned int poll)
@@ -169,8 +174,14 @@ void KBiff::setMailboxList(const QList<KBiffMailbox>& mailbox_list, unsigned int
         connect(monitor, SIGNAL(signal_currentStatus(const int, const QString&, const KBiffMailState)),
                 this, SLOT(currentStatus(const int, const QString&, const KBiffMailState)));
         connect(monitor, SIGNAL(signal_noMail()), this, SLOT(displayPixmap()));
+        connect(monitor, SIGNAL(signal_noMail()),
+                this, SLOT(haveNoNewMail()));
         connect(monitor, SIGNAL(signal_oldMail()), this, SLOT(displayPixmap()));
+        connect(monitor, SIGNAL(signal_oldMail()),
+                this, SLOT(haveNoNewMail()));
         connect(monitor, SIGNAL(signal_noConn()), this, SLOT(displayPixmap()));
+        connect(monitor, SIGNAL(signal_noConn()),
+                this, SLOT(haveNoNewMail()));
         connect(monitor, SIGNAL(signal_invalidLogin(const QString&)),
                 this, SLOT(invalidLogin(const QString&)));
         connect(monitor, SIGNAL(signal_fetchMail(const QString&)),
@@ -180,7 +191,7 @@ void KBiff::setMailboxList(const QList<KBiffMailbox>& mailbox_list, unsigned int
     myMUTEX = false;
 }
 
-const bool KBiff::isDocked() const
+bool KBiff::isDocked() const
 {
     return docked;
 }
@@ -202,7 +213,7 @@ void KBiff::readSessionConfig()
 ///////////////////////////////////////////////////////////////////////////
 // Protected Virtuals
 ///////////////////////////////////////////////////////////////////////////
-void KBiff::mousePressEvent(QMouseEvent *event)
+void KBiff::mousePressEvent(QMouseEvent *e)
 {
     // regardless of which button, get rid of the status box
     if (status)
@@ -217,7 +228,7 @@ void KBiff::mousePressEvent(QMouseEvent *event)
     }
 
     // check if this is a right click
-    if(event->button() == RightButton)
+    if(e->button() == RightButton)
     {
         // popup the context menu
         popupMenu();
@@ -322,11 +333,11 @@ bool KBiff::isGIF8x(const QString& file_name)
 
     /* Now test for the magical GIF8(9|7a) */
     if (header[0] == 'G' &&
-         header[1] == 'I' &&
-         header[2] == 'F' &&
-         header[3] == '8' &&
-         header[4] == '9' ||
-        (header[4] == '7' && header[5] == 'a'))
+        header[1] == 'I' &&
+        header[2] == 'F' &&
+        header[3] == '8' &&
+       (header[4] == '9' || (header[4] == '7' &&
+                             header[5] == 'a')))
     {
         /* Success! */
         return true;
@@ -394,14 +405,32 @@ void KBiff::displayPixmap()
         }
     }
 
-    if (has_new)
+    if ( !isRunning() )
+    {
+        pixmap_name = stoppedIcon;
+        mled->Off();
+    }
+    else if (has_new)
+    {
         pixmap_name = newMailIcon;
+        // turn on led for new mail, otherwise turn off
+        mled->On();
+    }
     else if (has_old)
+    {
         pixmap_name = oldMailIcon;
+        mled->Off();
+    }
     else if (has_noconn)
+    {
         pixmap_name = noConnIcon;
+        mled->Off();
+    }
     else
+    {
         pixmap_name = noMailIcon;
+        mled->Off();
+    }
 
     if (docked)
     {
@@ -441,15 +470,15 @@ void KBiff::currentStatus(const int num, const QString& the_mailbox, const KBiff
   statusChanged = true;
   // iterate through all saved notify dialogs to see if "our" one is
   // currently being displayed
-  KBiffNotify *notify;
-  for (notify = notifyList.first();
-       notify != 0;
-       notify = notifyList.next())
+  KBiffNotify *notifyptr;
+  for (notifyptr = notifyList.first();
+       notifyptr != 0;
+       notifyptr = notifyList.next())
   {
     // if this one is not visible, delete it from the list.  the only
     // way it will again become visible is if the haveNewMail slot
     // gets triggered
-    if (notify->isVisible() == false)
+    if (notifyptr->isVisible() == false)
     {
       notifyList.remove();
     }
@@ -457,7 +486,7 @@ void KBiff::currentStatus(const int num, const QString& the_mailbox, const KBiff
     {
       // if this box is visible (active), we see if it is the one
       // we are looking for
-      if (notify->getMailbox() == the_mailbox)
+      if (notifyptr->getMailbox() == the_mailbox)
       {
         // yep.  now, if there is new mail, we set the new number in
         // the dialog.  if it is any other state, we remove this
@@ -465,7 +494,7 @@ void KBiff::currentStatus(const int num, const QString& the_mailbox, const KBiff
         switch (the_state)
         {
           case NewMail:
-            notify->setNew(num);
+            notifyptr->setNew(num);
             break;
           case OldMail:
           case NoMail:
@@ -513,9 +542,24 @@ void KBiff::haveNewMail(const int num, const QString& the_mailbox)
         notify_dlg->show();
 
         // half-hearted attempt to center this
-        int x = (KApplication::desktop()->width() - notify_dlg->width()) / 2;
-        int y = (KApplication::desktop()->height() - notify_dlg->height()) / 2;
-        notify_dlg->move(x, y);
+        int x_pos = (KApplication::desktop()->width() - notify_dlg->width()) / 2;
+        int y_pos = (KApplication::desktop()->height() - notify_dlg->height()) / 2;
+        notify_dlg->move(x_pos, y_pos);
+    }
+}
+
+void KBiff::haveNoNewMail()
+{
+    displayPixmap();
+
+    // run a command if we have to
+    if (runResetCommand)
+    {
+        // make sure the command exists
+        if (!runResetCommandPath.isEmpty())
+        {
+            executeCommand(runResetCommandPath);
+        }
     }
 }
 
@@ -655,6 +699,7 @@ void KBiff::stop()
     {
         monitor->stop();
     }
+    displayPixmap();
 }
 
 void KBiff::start()
@@ -730,6 +775,7 @@ void KBiff::reset()
     newMailIcon = "newmail";
     oldMailIcon = "oldmail";
     noConnIcon  = "noconn";
+    stoppedIcon = "stopped";
 
     docked    = false;
     isSecure  = false;
