@@ -1,11 +1,11 @@
 /*
  * kbiff.h
- * Copyright (C) 1999 Kurt Granroth <granroth@kde.org>
+ * Copyright (C) 1999-2004 Kurt Granroth <granroth@kde.org>
  *
  * This file contains the declaration of the main KBiff
  * widget.
  *
- * $Id: kbiff.h,v 1.5 2001/10/22 22:35:29 gran Exp $
+ * $Id$
  */
 #ifndef KBIFF_H 
 #define KBIFF_H 
@@ -22,12 +22,6 @@
 #include <kbiffurl.h>
 #include <kapp.h>
 
-// mediatool.h is needed by kaudio.h
-//extern "C" {
-//#include <mediatool.h>
-//} 
-//#include <kaudio.h>
-
 #include <qlabel.h>
 #include <qtimer.h>
 
@@ -37,14 +31,16 @@
 #include <dcopclient.h>
 #include <dcopobject.h>
 
+class Led;
+
 class KBiff : public QLabel, virtual public DCOPObjectProxy
 {
     Q_OBJECT
 public:
-    KBiff(DCOPClient *client, QWidget *parent = 0);
+    KBiff(DCOPClient *client_, QWidget *parent_ = 0);
     virtual ~KBiff();
 
-    const bool isDocked() const;
+    bool isDocked() const;
 
     void processSetup(const KBiffSetup* setup, bool start);
     void readSessionConfig();
@@ -78,6 +74,7 @@ protected slots:
     void invokeHelp();
     void displayPixmap();
     void haveNewMail(const int, const QString &);
+    void haveNoNewMail();
     void currentStatus(const int, const QString &, const KBiffMailState);
     void dock();
     void setup();
@@ -115,6 +112,8 @@ protected:
     bool    systemBeep;
     bool    runCommand;
     QString runCommandPath;
+    bool    runResetCommand;
+    QString runResetCommandPath;
     bool    playSound;
     QString playSoundPath;
     bool    notify;
@@ -130,9 +129,12 @@ protected:
     QString newMailIcon;
     QString oldMailIcon;
     QString noConnIcon;
+    QString stoppedIcon;
 
     KBiffStatus *status;
     bool         statusChanged;
+
+    Led *mled;
 };
 
 #endif // KBIFF_H 
