@@ -1,13 +1,12 @@
 /*
  * kbiffmonitor.cpp
- * Copyright (C) 1999-2002 Kurt Granroth <granroth@kde.org>
+ * Copyright (C) 1999-2008 Kurt Granroth <granroth@kde.org>
  *
  * This file contains the implementation of KBiffMonitor and
  * associated classes.
- *
- * $Id: kbiffmonitor.cpp,v 1.13 2002/11/02 03:36:35 gran Exp $
  */
 #include "kbiffmonitor.h"
+#include "kbiffmonitor.moc"
 
 #include <kmessagebox.h>
 
@@ -43,6 +42,10 @@
 #include <qdatetime.h>
 #include <ksimpleconfig.h>
 
+// Needed for CRAM-MD5 and APOP
+#include <kmdcodec.h>
+#include "kbiffcrypt.h"
+
 #define MAXSTR (1024)
 
 #define MAIL_STATE_FILE "kbiffstate"
@@ -261,13 +264,40 @@ void KBiffMonitor::setMailbox(KBiffURL& url)
         mailbox  = url.path().right(url.path().length() - 1); 
         port     = (url.port() > 0) ? url.port() : 143;
 
-        preauth = url.query() == "?preauth" ? true : false;
-        keepalive = url.ref() == "keepalive" ? true : false;
-        bool async = url.ref() == "async" ? true : false;
+        preauth = url.searchPar("preauth") == "yes";
+        keepalive = url.searchPar("keepalive") == "yes";
+        bool async = url.searchPar("async") == "yes";  
         imap->setAsync(async);
+#ifdef USE_SSL
+        imap->setSSL(false);
+#endif // USE_SSL
         simpleURL = "imap4://" + server + "/" + mailbox;
     }
 
+#ifdef USE_SSL
+    if (protocol == "imap4s")
+    {
+        disconnect(this);
+
+        imap = new KBiffImap;
+
+        connect(this, SIGNAL(signal_checkMail()), SLOT(checkImap()));
+        server   = url.host();
+        user     = url.user();
+        password = url.pass();
+
+        mailbox  = url.path().right(url.path().length() - 1); 
+        port     = (url.port() > 0) ? url.port() : 993;
+
+        preauth = url.searchPar("preauth") == "yes";
+        keepalive = url.searchPar("keepalive") == "yes";
+        bool async = url.searchPar("async") == "yes";
+        imap->setAsync(async);
+	imap->setSSL(true);
+        simpleURL = "imap4s://" + server + "/" + mailbox;
+    }
+#endif // USE_SSL
+
     if (protocol == "pop3")
     {
         disconnect(this);
@@ -281,13 +311,47 @@ void KBiffMonitor::setMailbox(KBiffURL& url)
         mailbox  = url.user();
         port     = (url.port() > 0) ? url.port() : 110;
 
-        keepalive = url.ref() == "keepalive" ? true : false;
-        bool async = url.ref() == "async" ? true : false;
+        keepalive = url.searchPar("keepalive") == "yes";
+        bool async = url.searchPar("async") == "yes";
         pop->setAsync(async);
+        // preserve existing behaviour, prior to adding disable apop,
+        // by setting Apop on, even if no apop parameter is found in the mailbox url
+        bool useApop = !( url.searchPar("apop") == "no" );
+        pop->setApop( useApop );
+#ifdef USE_SSL
+        pop->setSSL(false);
+#endif // USE_SSL
 
         simpleURL = "pop3://" + server + "/" + mailbox;
     }
 
+#ifdef USE_SSL
+    if (protocol == "pop3s")
+    {
+        disconnect(this);
+
+        pop = new KBiffPop;
+
+        connect(this, SIGNAL(signal_checkMail()), SLOT(checkPop()));
+        server   = url.host();
+        user     = url.user();
+        password = url.pass();
+        mailbox  = url.user();
+        port     = (url.port() > 0) ? url.port() : 995;
+
+        keepalive = url.searchPar("keepalive") == "yes";
+        bool async = url.searchPar("async") == "yes";
+        pop->setAsync(async);
+        // preserve existing behaviour, prior to adding disable apop,
+        // by setting Apop on, even if no apop parameter is found in the mailbox url
+        bool useApop = !( url.searchPar("apop") == "no" );
+        pop->setApop( useApop );
+        pop->setSSL(true);
+
+        simpleURL = "pop3s://" + server + "/" + mailbox;
+    }
+#endif // USE_SSL
+
     if (protocol == "mbox")
     {
         disconnect(this);
@@ -342,9 +406,12 @@ void KBiffMonitor::setMailbox(KBiffURL& url)
         mailbox  = url.path().right(url.path().length() - 1); 
         port     = (url.port() > 0) ? url.port() : 119;
 
-        keepalive = url.ref() == "keepalive" ? true : false;
-        bool async = url.ref() == "async" ? true : false;
+        keepalive = url.searchPar("keepalive") == "yes";
+        bool async = url.searchPar("async") == "yes";
         nntp->setAsync(async);
+#ifdef USE_SSL
+        nntp->setSSL(false);
+#endif // USE_SSL
         simpleURL = "nntp://" + server + "/" + mailbox;
     }
 
@@ -432,22 +499,9 @@ void KBiffMonitor::checkMbox()
         // read.
         {
             utimbuf buf;
-            // we use 'setTime_t' since it takes timezone into account
-            QDateTime unixEpoch;
-            unixEpoch.setTime_t(0);
-            // do a simple test for those systems with hosed timezones.  if
-            // the lastRead and calculated lastRead aren't the same, then
-            // something is wrong with the system
-            QDateTime calc;
-            calc.setTime_t(-mbox.lastRead().secsTo(unixEpoch));
-            if (mbox.lastRead() == calc)
-            {
-              // QDateTime::secsTo returns a negative value if the argument
-              // is before the time the object refers to.
-              buf.actime = -mbox.lastRead().secsTo(unixEpoch);
-              buf.modtime = -mbox.lastModified().secsTo(unixEpoch);
-              utime(QFile::encodeName(mailbox), &buf);
-            }
+            buf.actime = mbox.lastRead().toTime_t();
+            buf.modtime = mbox.lastModified().toTime_t();
+            utime(QFile::encodeName(mailbox), &buf);
         }
 
         // if there are any new messages, consider the state New
@@ -494,16 +548,14 @@ void KBiffMonitor::checkPop()
             return;
         }
 
-        command = "USER " + user + "\r\n";
-        if (pop->command(command) == false)
-        {
-            pop->close();
-            invalidLogin();
-            return;
-        }
-
-        command = "PASS " + password + "\r\n";
-        if (pop->command(command) == false)
+        // find out if APOP is supported
+        pop->parseBanner();
+        
+        // find other possibly useful capabilities
+        // we don't care if this fails
+        pop->command("CAPA\r\n");
+        
+        if (pop->authenticate(user, password) == false )
         {
             pop->close();
             invalidLogin();
@@ -568,20 +620,9 @@ void KBiffMonitor::checkImap()
         }
 
         do_login = true;
-    }
-    
-    // imap allows spaces in usernames... we need to take care of that
-    user = imap->mungeUserPass(user);
 
-    // also asterisks (*) in passwords. maybe it's a good idea
-    // to _always_ munge the user and the password.
-    password = imap->mungeUserPass(password);
-
-    // if we are preauthorized OR we want to keep the session alive, then
-    // we don't login.  Otherwise, we do.
-    if ((preauth == false) && (do_login == true))
-    {
-        command = QString().setNum(seq) + " LOGIN " + user + " " + password + "\r\n";
+        // check the server's capabilities (see RFC 3050, 6.1.1)
+        command = QString().setNum(seq) + " CAPABILITY\r\n";
         if (imap->command(command, seq) == false)
         {
             invalidLogin();
@@ -590,23 +631,25 @@ void KBiffMonitor::checkImap()
         seq++;
     }
 
+    // if we are preauthorized OR we want to keep the session alive, then
+    // we don't login.  Otherwise, we do.
+    if ((preauth == false) && (do_login == true))
+    {
+        if (imap->authenticate(&seq, user, password) == false)
+        {
+            invalidLogin();
+            return;
+        }
+    }
+
     // reset the numbers from the last check
     imap->resetNumbers();
-    // select the mailbox
-    command = QString().setNum(seq) + " EXAMINE " + mailbox + "\r\n";
-    if (imap->command(command, seq) == false)
-        return;
-    seq++;
-
-    // this gets the number of undeleted messages
-    command = QString().setNum(seq) + " SEARCH UNDELETED\r\n";
-    if (imap->command(command, seq) == false)
-        return;
-    seq++;
-    // this gets the number of undeleted _new_ messages
-    command = QString().setNum(seq) + " SEARCH UNDELETED UNSEEN\r\n";
-    if (imap->command(command, seq) == false)
+    
+    // The STATUS COMMAND is documented in RFC2060, 6.3.10
+    command = QString().setNum(seq) + " STATUS " + mailbox + " (UNSEEN MESSAGES)\r\n";
+    if ( ! imap->command(command, seq)) {
         return;
+    }
     seq++;
 
     // lets not logout if we want to keep the session alive
@@ -663,7 +706,7 @@ void KBiffMonitor::checkMaildir()
             // determining "new" (or "unread") mail in maildir folders
             // is a *little* tricky.  all mail in the 'new' folder are
             // new, of course... but so is all mail in the 'cur'
-            // folder that doesn't have a ':2,[R|S|T]' after it.
+            // folder that doesn't have a ':2,[F|R|S|T]' after it.
             newCount = new_mailbox.count();
             curCount = cur_mailbox.count();
 
@@ -671,7 +714,7 @@ void KBiffMonitor::checkMaildir()
             QFileInfoListIterator it(*cur_list);
             QFileInfo *info;
 
-            static QRegExp suffix(":2,?R?S?T?$");
+            static QRegExp suffix(":2,?F?R?S?T?$");
             while ((info = it.current()))
             {
                 if (info->fileName().findRev(suffix) == -1)
@@ -776,7 +819,7 @@ void KBiffMonitor::checkNntp()
         char *buffer = c_buffer;
 
         // skip over the mailbox name
-        for(; buffer && *buffer != ' '; buffer++);
+        for(; buffer && *buffer != ' '; buffer++) {}
 
         // iterate over the sequence until we hit a newline or end of string
         while (buffer && *buffer != '\n' && *buffer != '\0')
@@ -814,7 +857,7 @@ void KBiffMonitor::checkNntp()
             last = current;
 
             // skip over all of these digits
-            for(;buffer && isdigit(*buffer); buffer++);
+            for(;buffer && isdigit(*buffer); buffer++) {}
 
             // is this a range?
             if (*buffer == '-')
@@ -869,7 +912,7 @@ void KBiffMonitor::checkMHdir()
                     // read till the end of the line
 
                     int c;
-                    while((c=mhseq.getch()) >=0 && c !='\n');
+                    while((c=mhseq.getch()) >=0 && c !='\n') {}
                 }
                 if (!strncmp(buffer, "unseen:", 7))
                 {
@@ -914,7 +957,7 @@ void KBiffMonitor::checkMHdir()
                             }
 
                             // skip over all digits
-                            for(;buffer && isdigit(*buffer); buffer++);
+                            for(;buffer && isdigit(*buffer); buffer++) {}
 
                             // check if we are in a range
                             if (*buffer == '-')
@@ -1238,7 +1281,7 @@ int KBiffMonitor::mboxMessages()
 
             int c;
 
-            while((c=mbox.getch()) >=0 && c !='\n');
+            while((c=mbox.getch()) >=0 && c !='\n') {}
         }
 
         if (!in_header && real_from(buffer))
@@ -1337,6 +1380,9 @@ void KBiffMonitor::invalidLogin()
 // KBiffSocket
 ///////////////////////////////////////////////////////////////////////////
 KBiffSocket::KBiffSocket() : async(false), socketFD(-1), messages(0), newMessages(-1)
+#ifdef USE_SSL
+    , ssltunnel(0)
+#endif // USE_SSL
 {
     FD_ZERO(&socketFDS);
 
@@ -1351,6 +1397,13 @@ KBiffSocket::KBiffSocket() : async(false), socketFD(-1), messages(0), newMessage
 KBiffSocket::~KBiffSocket()
 {
     close();
+#ifdef USE_SSL
+    if (ssltunnel)
+    {
+        delete ssltunnel;
+        ssltunnel = 0;
+    }
+#endif // USE_SSL
 }
 
 int KBiffSocket::numberOfMessages()
@@ -1366,6 +1419,13 @@ int KBiffSocket::numberOfNewMessages()
 void KBiffSocket::close()
 {
 
+#ifdef USE_SSL
+    if (isSSL() && (socketFD != -1) && (ssltunnel != 0))
+    {
+        ssltunnel->close();
+    }
+#endif // USE_SSL
+
     if (socketFD != -1)
         ::close(socketFD);
 
@@ -1463,6 +1523,27 @@ bool KBiffSocket::connectSocket(const QString& host, unsigned short int port)
       
         // The connection has finished. Catch any error in a call to readLine()
     }
+    
+#ifdef USE_SSL
+    // Initialize SSL tunnel, if needed
+    if (isSSL())
+    {
+        if (ssltunnel == 0)
+            ssltunnel = new KSSL(true);
+	else
+	    ssltunnel->reInitialize();
+	if (ssltunnel == 0)
+	{
+            close();
+  	    return false;
+	}
+        if (ssltunnel->connect(socketFD) != 1)
+        {
+            close();
+  	    return false;
+        }
+    }
+#endif // USE_SSL
 
     // we're connected!  see if the connection is good
     QString line(readLine());
@@ -1475,6 +1556,7 @@ bool KBiffSocket::connectSocket(const QString& host, unsigned short int port)
     }
 
     // everything is swell
+    banner = line;  // save the banner for use by subclasses
     return true;
 }
 
@@ -1513,6 +1595,48 @@ void KBiffSocket::setAsync(bool on)
     }
 }
 
+#ifdef USE_SSL
+bool KBiffSocket::isSSL()
+{
+    return usessl;
+}
+
+void KBiffSocket::setSSL(bool on)
+{
+    if (usessl == on) return;
+    if (!KSSL::doesSSLWork())
+    {
+        usessl = false;
+        return;
+    }
+    usessl = on;
+    if (active())
+    {
+        switch (usessl)
+	{
+	    case false:
+	        ssltunnel->close();
+		delete ssltunnel;
+		ssltunnel = 0;
+		break;
+	    case true:
+	        if (ssltunnel == 0)
+		    ssltunnel = new KSSL(true);
+		else
+		    ssltunnel->reInitialize();
+		if (ssltunnel == 0)
+		{
+		    usessl = false;
+		    break;
+		}
+	        if (ssltunnel->connect(socketFD) != 1)
+		    usessl = false;
+		break;
+	}
+    }
+}
+#endif // USE_SSL
+
 int KBiffSocket::writeLine(const QString& line)
 {
     int bytes = 0;
@@ -1521,6 +1645,14 @@ int KBiffSocket::writeLine(const QString& line)
     if (!active())
         return -1;
 
+#ifdef USE_SSL
+    if (isSSL())
+    {
+        if ((bytes = ssltunnel->write(line.ascii(), line.length())) <= 0)
+	    close();
+    }
+    else
+#endif // USE_SSL
     if ((bytes = ::write(socketFD, line.ascii(), line.length())) <= 0)
         close();
 
@@ -1533,6 +1665,14 @@ QString KBiffSocket::readLine()
     char buffer;
     ssize_t bytes = -1;
 
+#ifdef USE_SSL
+    if (isSSL())
+    {
+        while (((bytes = ssltunnel->read(&buffer, 1)) > 0) && (buffer != '\n'))
+	    response += buffer;
+    }
+    else
+#endif // USE_SSL
     if (!async)
         while (((bytes = ::read(socketFD, &buffer, 1)) > 0) && (buffer != '\n'))
             response += buffer;
@@ -1568,6 +1708,12 @@ QString KBiffSocket::readLine()
 ///////////////////////////////////////////////////////////////////////////
 // KBiffImap
 ///////////////////////////////////////////////////////////////////////////
+KBiffImap::KBiffImap()
+{
+    /* Assume that the IMAP server does no fancy authentication */
+    auth_cram_md5 = false;
+}
+
 KBiffImap::~KBiffImap()
 {
     close();
@@ -1575,10 +1721,9 @@ KBiffImap::~KBiffImap()
 
 bool KBiffImap::command(const QString& line, unsigned int seq)
 {
-    int len, match;
-    bool unseen = false;
     QString messagesListString;
     QStringList messagesList;
+    bool tried_cram_md5; // are we trying CRAM-MD5 ?
 
     if (writeLine(line) <= 0)
     {
@@ -1590,6 +1735,16 @@ bool KBiffImap::command(const QString& line, unsigned int seq)
     ok.sprintf("%d OK", seq);
     bad.sprintf("%d BAD", seq);
     no.sprintf("%d NO", seq);
+    
+    // must be case insensitive
+    QRegExp status("\\* STATUS", FALSE);
+    QRegExp capability("\\* CAPABILITY", FALSE);
+    QRegExp cram_md5("AUTHENTICATE CRAM-MD5", FALSE);
+
+    // are we trying CRAM-MD5 ?
+    tried_cram_md5 = cram_md5.search(line)>=0;
+    cram_md5 = QRegExp("\\+ ([A-Za-z0-9+/=]+)");
+    
     while (!(response = readLine()).isNull())
     {
         // if an error has occurred, we get a null string in return
@@ -1602,19 +1757,51 @@ bool KBiffImap::command(const QString& line, unsigned int seq)
         if ((response.find(bad) > -1) || (response.find(no) > -1))
             break;
 
-        QRegExp unseen_re(" UNSEEN");
-        if (unseen_re.match(line) > -1)
-            unseen = true;
+        /* The STATUS response is documented in RFC2060, 6.3.10/7.2.4
+         * Briefly: the response depends on command and looks like
+         *     * STATUS "some-imap-folder" ( requested-info )
+         * for example:
+         * C: . STATUS "INBOX" (UNSEEN MESSAGES)
+         * S: * STATUS "INBOX" (UNSEEN 2 MESSAGES 3)
+         * S: . OK STATUS Completed
+         */
+        if (status.search(response) >= 0) {
+            QRegExp unseen("UNSEEN ([0-9]*)", FALSE);
+            if (unseen.search(response) >= 0) {
+                QString num = unseen.cap(1);
+                newMessages = num.toInt();
+            }
+            
+            QRegExp number("MESSAGES ([0-9]*)", FALSE);
+            if (number.search(response) >= 0) {
+                QString num = number.cap(1);
+                messages = num.toInt();
+            }
+        }
 
-        QRegExp messages_re("SEARCH [0-9 ]*");
-        if ((match = messages_re.match(response, 0, &len)) > -1)
-        {
-            messagesListString = response.mid(match + 7, len - 7);
-            messagesList = QStringList::split(' ', messagesListString);
-            if (unseen)
-                newMessages = messagesList.count();
-            else
-                messages = messagesList.count();
+        /* The CAPABILITY response is documented in RFC 3050,
+         * sections 6.1.1 and 7.2.1
+         * An example:
+         * C: . CAPABILITY
+         * S: * CAPABILITY IMAP4rev1 IDLE AUTH=PLAIN AUTH=CRAM-MD5
+         * S: . OK CAPABILITY completed.
+         */
+        if (capability.search(response) >= 0) {
+            QRegExp cram_md5_cap("AUTH=CRAM-MD5", FALSE);
+            if (cram_md5_cap.search(response) >= 0) {
+                auth_cram_md5 = true;
+            }
+        }
+
+        /* AUTHENTICATE CRAM-MD5 response is documented in
+         * RFC 3050 6.2.2 and RFC 2195
+         */
+        if (tried_cram_md5 && cram_md5.search(response)>=0) {
+            chall_cram_md5 = KCodecs::base64Decode(cram_md5.cap(1).local8Bit());
+            if (chall_cram_md5.isNull())
+                break;
+
+            return true;
         }
     }
 
@@ -1640,9 +1827,62 @@ void KBiffImap::resetNumbers()
     newMessages = 0;
 }
 
+bool KBiffImap::authenticate(int *pseq, const QString& user, const QString& pass)
+{
+    QString cmd, username, password;
+
+    // If CRAM-MD5 is available, use it.  It's the best we know.
+    // RFC 2195 defines the CRAM-MD5 authentication method
+    // also see RFC 3501 section 6.2.2 for the AUTHENTICATE command
+    if( auth_cram_md5 )
+    {
+        cmd = QString("%1 AUTHENTICATE CRAM-MD5\r\n").arg(*pseq);
+        if (command(cmd, *pseq) == false)
+        {
+            return false;
+        }
+
+        // calculate the real response to the challenge
+        QString response = user + " " + KBiffCrypt::hmac_md5(chall_cram_md5, pass);
+        response = KCodecs::base64Encode(response.latin1());
+
+        // send the response
+        if (command(response+"\r\n", *pseq) == false)
+        {
+            return false;
+        }
+
+        return true;
+    }
+    else // well, we tried, LOGIN is the best we can do
+    {
+        // imap allows spaces in usernames... we need to take care of that
+        username = mungeUserPass(user);
+
+        // also asterisks (*) in passwords. maybe it's a good idea
+        // to _always_ munge the user and the password.
+        password = mungeUserPass(pass);
+
+        cmd = QString().setNum(*pseq) + " LOGIN "
+            + username + " "
+            + password + "\r\n";
+        if (command(cmd, *pseq) == false)
+        {
+            return false;
+        }
+        (*pseq)++;
+    }
+
+    return true;
+}
+
 ///////////////////////////////////////////////////////////////////////////
 // KBiffPop
 ///////////////////////////////////////////////////////////////////////////
+KBiffPop::KBiffPop() : use_apop( true )
+{
+}
+
 KBiffPop::~KBiffPop()
 {
     close();
@@ -1654,6 +1894,11 @@ void KBiffPop::close()
     KBiffSocket::close();
 }
 
+void KBiffPop::setApop( bool enabled )
+{
+    use_apop = enabled;
+}
+
 bool KBiffPop::command(const QString& line)
 {
     if (writeLine(line) <= 0)
@@ -1691,7 +1936,7 @@ bool KBiffPop::command(const QString& line)
     {
         for (messages = 0, response = readLine();
              !response.isNull() && response.left(1) != ".";
-             messages++, response = readLine());
+             messages++, response = readLine()) {}
     }
     else
     if (line == "STAT\r\n")
@@ -1699,6 +1944,34 @@ bool KBiffPop::command(const QString& line)
         if (!response.isNull())
             sscanf(response.ascii(), "+OK %d", &messages);
     }
+    else
+    // find out what the server is capable of
+    if (line == "CAPA\r\n")
+    {
+        QRegExp rx("\\bCRAM-MD5\\b");
+
+        auth_cram_md5 = false;  // assume no support
+
+        for (response = readLine();
+             !response.isNull() && response.left(1) != ".";
+             response = readLine())
+        {
+            if (response.left(4) == "SASL")
+                auth_cram_md5 = response.find(rx) != -1;
+        }
+    }
+    else
+    // look for the CRAM-MD5 challenge
+    if (line == "AUTH CRAM-MD5\r\n")
+    {
+        QRegExp challenge("\\+ ([A-Za-z0-9+/=]+)");
+        if (challenge.search(response) == -1 )
+        {
+            return false;
+        }
+
+        chall_cram_md5 = KCodecs::base64Decode(challenge.cap(1).local8Bit()); 
+    }
 
     return !response.isNull();
 }
@@ -1708,6 +1981,99 @@ KBiffUidlList KBiffPop::getUidlList() const
     return uidlList;
 }
 
+/*!
+   This method parses the initial response from the POP3 server.
+   The response is defined in RFC 1939 sections 4 and 7.
+
+    \fn KBiffPop::parse_banner(void)
+ */
+bool KBiffPop::parseBanner(void)
+{
+    // RFC 1939 section 3 says server MUST use uppercase
+    if( banner.left(3) != "+OK" ) {
+        auth_apop = false;
+        return false;
+    }
+
+    // Look for the banner part that indicates APOP support
+    QRegExp rx("(<[a-zA-Z0-9_+.-]+@[a-zA-Z0-9_+.-]+>)");
+    if( rx.search(banner) == -1 || !use_apop ) {
+        auth_apop = false;
+    } else {
+        chall_apop = rx.cap(1).latin1();
+        auth_apop  = true;
+    }
+
+    return true;
+}
+
+/*!
+    This method authenticates using the most secure
+    technique available.
+    \fn KBiffPop::authenticate(const QString& user, const QString& pass)
+ */
+bool KBiffPop::authenticate(const QString& user, const QString& pass)
+{
+    QString popcommand;
+
+    // CRAM-MD5 authentication is the most secure we can handle
+    // the use of the AUTH command is documented in RFC 1734
+    if( auth_cram_md5 )
+    {
+        if (this->command("AUTH CRAM-MD5\r\n") == false)
+        {
+            return false;
+        }
+
+        // calculate the real response to the challenge
+        QString response = user + " " + KBiffCrypt::hmac_md5(chall_cram_md5, pass);
+        response = KCodecs::base64Encode(response.latin1());
+
+        // send the response
+        if (this->command(response+"\r\n") == false)
+        {
+            return false;
+        }
+        
+        return true;
+    }
+
+    // APOP is not as secure as CRAM-MD5 but it's still better
+    // than sending the password in the clear
+    if( auth_apop )
+    {
+        QCString digest;
+
+        KMD5 md5(chall_apop);
+        md5.update(pass);
+
+        digest = md5.hexDigest();
+        
+        popcommand = QString("APOP %1 %2\r\n").arg(user, digest.data());
+        if (this->command(popcommand) == false)
+        {
+            return false;
+        }
+        
+        return true;
+    }
+    
+    // lastly we'll try regular, plain-text authentication
+    
+    popcommand = "USER " + user + "\r\n";
+    if (this->command(popcommand) == false)
+    {
+        return false;
+    }
+
+    popcommand = "PASS " + pass + "\r\n";
+    if (this->command(popcommand) == false)
+    {
+        return false;
+    }
+    
+    return true;
+}
 ///////////////////////////////////////////////////////////////////////////
 // KBiffNntp
 ///////////////////////////////////////////////////////////////////////////
