@@ -1,4 +1,5 @@
 #include "kbiff.h"
+#include "version.h"
 #include <kwin.h>
 #include <kapp.h>
 #include <klocale.h>
@@ -10,8 +11,6 @@
 static const char *description = 
   I18N_NOOP("Full featured mail notification utility.");
 
-static const char *version = "3.6.3";
-
 static KCmdLineOptions option[] =
 {
    { "secure", I18N_NOOP("Run in secure mode"), 0 },
@@ -19,11 +18,11 @@ static KCmdLineOptions option[] =
    { 0, 0, 0 }
 };
 
-int main(int argc, char *argv[])
+extern "C" KDE_EXPORT int kdemain(int argc, char *argv[])
 {
   KAboutData aboutData( "kbiff", I18N_NOOP("KBiff"),
-      version, description, KAboutData::License_GPL,
-      "(c) 1998-2002, Kurt Granroth");
+      kbiff_version, description, KAboutData::License_GPL,
+      "(c) 1998-2008, Kurt Granroth");
   aboutData.addAuthor("Kurt Granroth",0, "granroth@kde.org");
   KCmdLineArgs::init( argc, argv, &aboutData );
   KCmdLineArgs::addCmdLineOptions(option);
