@@ -5,7 +5,7 @@
  * This file contains the declaration of the KBiffNotify
  * widget.
  *
- * $Id: notify.h,v 1.2 2001/03/24 18:50:21 gran Exp $
+ * $Id$
  */
 #ifndef KBIFFNOTIFY_H 
 #define KBIFFNOTIFY_H 
@@ -23,11 +23,11 @@ class KBiffNotify : public QDialog
 {
 	Q_OBJECT
 public:
-	KBiffNotify(const int num_new, const QString& mailbx);
+	KBiffNotify(QWidget *parent, const int num_new, const QString& mailbx);
 	virtual ~KBiffNotify();
 
 	const QString getMailbox() { return mailbox; }
-	const int newMessages() { return messages; }
+	int newMessages() { return messages; }
 
 	void setNew(const int num_new);
 
