@@ -1,13 +1,12 @@
 /*
  * setupdlg.cpp
- * Copyright (C) 1999-2002 Kurt Granroth <granroth@kde.org>
+ * Copyright (C) 1999-2008 Kurt Granroth <granroth@kde.org>
  *
  * This file contains the implementation of the setup dialog
  * class for KBiff.
- *
- * $Id: setupdlg.cpp,v 1.10 2002/11/02 03:36:35 gran Exp $
  */
 #include "setupdlg.h"
+#include "setupdlg.moc"
 
 #include <qgroupbox.h>
 #include <qfileinfo.h>
@@ -45,6 +44,7 @@
 #include <kwin.h>
 
 #include <kbiffcodec.h>
+#include "version.h"
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -64,7 +64,7 @@
 ///////////////////////////////////////////////////////////////////////////
 // KBiffSetup
 ///////////////////////////////////////////////////////////////////////////
-KBiffSetup::KBiffSetup(const QString& profile, bool secure)
+KBiffSetup::KBiffSetup(const QString& profile_, bool secure_)
     : KDialog(0, 0, true)
 {
   // set the icon just to be cute
@@ -72,10 +72,10 @@ KBiffSetup::KBiffSetup(const QString& profile, bool secure)
 
   // make sure the profile is *something*
   QString the_profile;
-  if (profile.isEmpty() || profile.isNull())
+  if (profile_.isEmpty() || profile_.isNull())
     the_profile = getSomeProfile();
   else
-    the_profile = profile;
+    the_profile = profile_;
 
   setCaption(i18n("KBiff Setup"));
 
@@ -148,7 +148,7 @@ KBiffSetup::KBiffSetup(const QString& profile, bool secure)
   connect(cancel_button, SIGNAL(clicked()), SLOT(reject()));
 
   // are we secure?
-  isSecure = secure;
+  isSecure = secure_;
 
   // NOW, SETUP ALL THE LAYOUTS!
   // This layout handles the buttons for the profile combobox
@@ -186,7 +186,7 @@ KBiffSetup::~KBiffSetup()
 {
 }
 
-const bool KBiffSetup::getSecure() const
+bool KBiffSetup::getSecure() const
 {
   return isSecure;
 }
@@ -231,22 +231,27 @@ const QString KBiffSetup::getNoConnIcon() const
   return generalTab->getButtonNoConn();
 }
 
-const bool KBiffSetup::getSessionManagement() const
+const QString KBiffSetup::getStoppedIcon() const
+{
+  return generalTab->getButtonStopped();
+}
+
+bool KBiffSetup::getSessionManagement() const
 {
   return generalTab->getSessionManagement();
 }
 
-const bool KBiffSetup::getCheckStartup() const
+bool KBiffSetup::getCheckStartup() const
 {
   return generalTab->getCheckStartup();
 }
 
-const bool KBiffSetup::getDock() const
+bool KBiffSetup::getDock() const
 {
   return generalTab->getDock();
 }
 
-const unsigned int KBiffSetup::getPoll() const
+unsigned int KBiffSetup::getPoll() const
 {
   return generalTab->getPoll();
 }
@@ -256,32 +261,42 @@ const QString KBiffSetup::getRunCommandPath() const
   return newmailTab->getRunCommandPath();
 }
 
+const QString KBiffSetup::getRunResetCommandPath() const
+{
+  return newmailTab->getRunResetCommandPath();
+}
+
 const QString KBiffSetup::getPlaySoundPath() const
 {
   return newmailTab->getPlaySoundPath();
 }
 
-const bool KBiffSetup::getRunCommand() const
+bool KBiffSetup::getRunCommand() const
 {
   return newmailTab->getRunCommand();
 }
 
-const bool KBiffSetup::getPlaySound() const
+bool KBiffSetup::getRunResetCommand() const
+{
+  return newmailTab->getRunResetCommand();
+}
+
+bool KBiffSetup::getPlaySound() const
 {
   return newmailTab->getPlaySound();
 }
 
-const bool KBiffSetup::getSystemBeep() const
+bool KBiffSetup::getSystemBeep() const
 {
   return newmailTab->getSystemBeep();
 }
 
-const bool KBiffSetup::getNotify() const
+bool KBiffSetup::getNotify() const
 {
   return newmailTab->getNotify();
 }
 
-const bool KBiffSetup::getStatus() const
+bool KBiffSetup::getStatus() const
 {
   return newmailTab->getStatus();
 }
@@ -291,7 +306,7 @@ void KBiffSetup::invokeHelp()
   kapp->invokeHelp();
 }
 
-void KBiffSetup::readConfig(const QString& profile)
+void KBiffSetup::readConfig(const QString& profile_)
 {
   QStringList profile_list;
 
@@ -317,7 +332,7 @@ void KBiffSetup::readConfig(const QString& profile)
     // read in the data from the first mailbox if we don't have a name
     for (int i = 0; i < comboProfile->count(); i++)
     {
-      if (QString(profile) == comboProfile->text(i))
+      if (QString(profile_) == comboProfile->text(i))
       {
         comboProfile->setCurrentItem(i);
         break;
@@ -325,7 +340,7 @@ void KBiffSetup::readConfig(const QString& profile)
     }
   }
   else
-    comboProfile->insertItem(profile);
+    comboProfile->insertItem(profile_);
 }
 
 QString KBiffSetup::getSomeProfile() const
@@ -499,8 +514,8 @@ void KBiffSetup::slotDeleteProfile()
 ///////////////////////////////////////////////////////////////////////
 // KBiffGeneralTab
 ///////////////////////////////////////////////////////////////////////
-KBiffGeneralTab::KBiffGeneralTab(const QString& profile, QWidget *parent)
-  : QWidget(parent)
+KBiffGeneralTab::KBiffGeneralTab(const QString& profile_, QWidget *parent_)
+  : QWidget(parent_)
 {
   // the poll time (in seconds)
   QLabel* poll_label = new QLabel(i18n("P&oll (sec):"), this);
@@ -546,6 +561,13 @@ KBiffGeneralTab::KBiffGeneralTab(const QString& profile, QWidget *parent)
   // group box to hold the icons together
   QGroupBox* icons_groupbox = new QGroupBox(i18n("Icons:"), this);
 
+  // "stopped" pixmap button
+  QLabel* stopped_label = new QLabel(i18n("&Stopped:"), icons_groupbox);
+  buttonStopped = new KIconButton(icons_groupbox);
+  buttonStopped->setFixedSize(50, 50);
+  buttonStopped->setIconType(KIcon::User, KIcon::Any, true);
+  stopped_label->setBuddy(buttonStopped);
+
   // "no mailbox" pixmap button
   QLabel* noconn_label = new QLabel(i18n("No Mail&box:"), icons_groupbox);
   buttonNoConn = new KIconButton(icons_groupbox);
@@ -587,6 +609,10 @@ KBiffGeneralTab::KBiffGeneralTab(const QString& profile, QWidget *parent)
   info_layout->addMultiCellWidget(checkNoStartup, 4, 4, 1, 2);
 
   // icons layout
+  QVBoxLayout *stopped_layout = new QVBoxLayout;
+  stopped_layout->addWidget(stopped_label);
+  stopped_layout->addWidget(buttonStopped);
+
   QVBoxLayout *no_conn_layout = new QVBoxLayout;
   no_conn_layout->addWidget(noconn_label);
   no_conn_layout->addWidget(buttonNoConn);
@@ -605,6 +631,8 @@ KBiffGeneralTab::KBiffGeneralTab(const QString& profile, QWidget *parent)
 
   QHBoxLayout *inner_icon_layout = new QHBoxLayout;
   inner_icon_layout->addStretch(1);
+  inner_icon_layout->addLayout(stopped_layout);
+  inner_icon_layout->addStretch(1);
   inner_icon_layout->addLayout(no_conn_layout);
   inner_icon_layout->addStretch(1);
   inner_icon_layout->addLayout(no_mail_layout);
@@ -627,24 +655,24 @@ KBiffGeneralTab::KBiffGeneralTab(const QString& profile, QWidget *parent)
   top_layout->addStretch(1);
 
   // now read in the config data for this profile
-  readConfig(profile);
+  readConfig(profile_);
 }
 
 KBiffGeneralTab::~KBiffGeneralTab()
 {
 }
 
-const bool KBiffGeneralTab::getSessionManagement() const
+bool KBiffGeneralTab::getSessionManagement() const
 {
   return checkNoSession->isChecked();
 }
 
-const bool KBiffGeneralTab::getCheckStartup() const
+bool KBiffGeneralTab::getCheckStartup() const
 {
   return checkNoStartup->isChecked();
 }
 
-const bool KBiffGeneralTab::getDock() const
+bool KBiffGeneralTab::getDock() const
 {
   return checkDock->isChecked();
 }
@@ -669,22 +697,27 @@ const QString KBiffGeneralTab::getButtonNoConn() const
   return buttonNoConn->icon();
 }
 
+const QString KBiffGeneralTab::getButtonStopped() const
+{
+  return buttonStopped->icon();
+}
+
 const QString KBiffGeneralTab::getMailClient() const
 {
   return editCommand->text();
 }
 
-const int KBiffGeneralTab::getPoll() const
+int KBiffGeneralTab::getPoll() const
 {
   return QString(editPoll->text()).toInt();
 }
 
-void KBiffGeneralTab::readConfig(const QString& profile)
+void KBiffGeneralTab::readConfig(const QString& profile_)
 {
   // open the config file
   KSimpleConfig *config = new KSimpleConfig(CONFIG_FILE, true);
 
-  config->setGroup(profile);
+  config->setGroup(profile_);
 
   // someday, we need to read in the system email prefs
   editPoll->setText(config->readEntry("Poll", "60"));
@@ -693,20 +726,18 @@ void KBiffGeneralTab::readConfig(const QString& profile)
   checkNoSession->setChecked(config->readBoolEntry("Sessions", true));
   checkNoStartup->setChecked(config->readBoolEntry("DontCheck", false));
 
-  QString no_mail, old_mail, new_mail, no_conn;
+  QString stopped, no_mail, old_mail, new_mail, no_conn;
+  stopped  = config->readEntry("StoppedPixmap", "stopped");
   no_mail  = config->readEntry("NoMailPixmap", "nomail");
   old_mail = config->readEntry("OldMailPixmap", "oldmail");
   new_mail = config->readEntry("NewMailPixmap", "newmail");
   no_conn  = config->readEntry("NoConnPixmap", "noconn");
 
   buttonOldMail->setIcon(old_mail);
-  buttonOldMail->setPixmap(UserIcon(old_mail));
   buttonNewMail->setIcon(new_mail);
-  buttonNewMail->setPixmap(UserIcon(new_mail));
   buttonNoMail->setIcon(no_mail);
-  buttonNoMail->setPixmap(UserIcon(no_mail));
   buttonNoConn->setIcon(no_conn);
-  buttonNoConn->setPixmap(UserIcon(no_conn));
+  buttonStopped->setIcon(stopped);
 
   delete config;
 }
@@ -745,12 +776,12 @@ static QString justIconName(const QString& icon_name)
   return info.baseName();
 }
 
-void KBiffGeneralTab::saveConfig(const QString& profile)
+void KBiffGeneralTab::saveConfig(const QString& profile_)
 {
   // open the config file for writing
   KSimpleConfig *config = new KSimpleConfig(CONFIG_FILE);
 
-  config->setGroup(profile);
+  config->setGroup(profile_);
 
   config->writeEntry("Poll", editPoll->text());
   config->writeEntry("MailClient", editCommand->text());
@@ -761,20 +792,26 @@ void KBiffGeneralTab::saveConfig(const QString& profile)
   config->writeEntry("NewMailPixmap", justIconName(buttonNewMail->icon()));
   config->writeEntry("OldMailPixmap", justIconName(buttonOldMail->icon()));
   config->writeEntry("NoConnPixmap", justIconName(buttonNoConn->icon()));
+  config->writeEntry("StoppedPixmap", justIconName(buttonStopped->icon()));
   delete config;
 }
 
 ///////////////////////////////////////////////////////////////////////
 // KBiffNewMailTab
 ///////////////////////////////////////////////////////////////////////
-KBiffNewMailTab::KBiffNewMailTab(const QString& profile, QWidget *parent)
-  : QWidget(parent)
+KBiffNewMailTab::KBiffNewMailTab(const QString& profile_, QWidget *parent_)
+  : QWidget(parent_)
 {
   // setup the Run Command stuff
   checkRunCommand = new QCheckBox(i18n("R&un Command"), this);
   editRunCommand = new QLineEdit(this);
   buttonBrowseRunCommand = new QPushButton(i18n("Browse"), this);
 
+  // setup the Run Reset Command stuff
+  checkRunResetCommand = new QCheckBox(i18n("R&un Reset-Command"), this);
+  editRunResetCommand = new QLineEdit(this);
+  buttonBrowseRunResetCommand = new QPushButton(i18n("Browse"), this);
+
   // setup the Play Sound stuff
   checkPlaySound = new QCheckBox(i18n("&Play Sound"), this);
   editPlaySound = new QLineEdit(this);
@@ -791,15 +828,21 @@ KBiffNewMailTab::KBiffNewMailTab(const QString& profile, QWidget *parent)
   // connect some slots and signals
   connect(buttonBrowsePlaySound, SIGNAL(clicked()), SLOT(browsePlaySound()));
   connect(buttonBrowseRunCommand, SIGNAL(clicked()), SLOT(browseRunCommand()));
+  connect(buttonBrowseRunResetCommand, SIGNAL(clicked()), SLOT(browseRunResetCommand()));
   connect(checkPlaySound, SIGNAL(toggled(bool)), SLOT(enablePlaySound(bool)));
   connect(buttonTestPlaySound, SIGNAL(clicked()), SLOT(testPlaySound()));
   connect(checkRunCommand, SIGNAL(toggled(bool)), SLOT(enableRunCommand(bool)));
+  connect(checkRunResetCommand, SIGNAL(toggled(bool)), SLOT(enableRunResetCommand(bool)));
 
   // NOW DO THE LAYOUT
   QHBoxLayout *run_command_layout = new QHBoxLayout(5);
   run_command_layout->addWidget(editRunCommand);
   run_command_layout->addWidget(buttonBrowseRunCommand);
 
+  QHBoxLayout *run_reset_command_layout = new QHBoxLayout(5);
+  run_reset_command_layout->addWidget(editRunResetCommand);
+  run_reset_command_layout->addWidget(buttonBrowseRunResetCommand);
+
   QHBoxLayout *play_sound_layout = new QHBoxLayout(5);
   play_sound_layout->addWidget(buttonTestPlaySound, 0);
   play_sound_layout->addWidget(editPlaySound, 1);
@@ -809,6 +852,9 @@ KBiffNewMailTab::KBiffNewMailTab(const QString& profile, QWidget *parent)
   top_layout->addWidget(checkRunCommand);
   top_layout->addLayout(run_command_layout);
 
+  top_layout->addWidget(checkRunResetCommand);
+  top_layout->addLayout(run_reset_command_layout);
+
   top_layout->addWidget(checkPlaySound);
   top_layout->addLayout(play_sound_layout);
 
@@ -817,7 +863,7 @@ KBiffNewMailTab::KBiffNewMailTab(const QString& profile, QWidget *parent)
   top_layout->addWidget(checkStatus);
   top_layout->addStretch(1);
 
-  readConfig(profile);
+  readConfig(profile_);
 }
 
 KBiffNewMailTab::~KBiffNewMailTab()
@@ -829,45 +875,50 @@ void KBiffNewMailTab::testPlaySound()
   KAudioPlayer::play(getPlaySoundPath());
 }
 
-void KBiffNewMailTab::readConfig(const QString& profile)
+void KBiffNewMailTab::readConfig(const QString& profile_)
 {
   // open the config file
   KSimpleConfig *config = new KSimpleConfig(CONFIG_FILE, true);
 
-  config->setGroup(profile);
+  config->setGroup(profile_);
 
   checkRunCommand->setChecked(config->readBoolEntry("RunCommand", false));
+  checkRunResetCommand->setChecked(config->readBoolEntry("RunResetCommand", false));
   checkPlaySound->setChecked(config->readBoolEntry("PlaySound", false));
   checkBeep->setChecked(config->readBoolEntry("SystemBeep", true));
   checkNotify->setChecked(config->readBoolEntry("Notify", true));
   checkStatus->setChecked(config->readBoolEntry("Status", true));
   editRunCommand->setText(config->readEntry("RunCommandPath"));
+  editRunResetCommand->setText(config->readEntry("RunResetCommandPath"));
   editPlaySound->setText(config->readEntry("PlaySoundPath"));
 
   enableRunCommand(checkRunCommand->isChecked());
+  enableRunResetCommand(checkRunResetCommand->isChecked());
   enablePlaySound(checkPlaySound->isChecked());
 
   delete config;
 }
 
-void KBiffNewMailTab::saveConfig(const QString& profile)
+void KBiffNewMailTab::saveConfig(const QString& profile_)
 {
   KSimpleConfig *config = new KSimpleConfig(CONFIG_FILE);
 
-  config->setGroup(profile);
+  config->setGroup(profile_);
 
   config->writeEntry("RunCommand", checkRunCommand->isChecked());
+  config->writeEntry("RunResetCommand", checkRunResetCommand->isChecked());
   config->writeEntry("PlaySound", checkPlaySound->isChecked());
   config->writeEntry("SystemBeep", checkBeep->isChecked());
   config->writeEntry("Notify", checkNotify->isChecked());
   config->writeEntry("Status", checkStatus->isChecked());
   config->writeEntry("RunCommandPath", editRunCommand->text());
+  config->writeEntry("RunResetCommandPath", editRunResetCommand->text());
   config->writeEntry("PlaySoundPath", editPlaySound->text());
 
   delete config;
 }
 
-const bool KBiffNewMailTab::getRunCommand() const
+bool KBiffNewMailTab::getRunCommand() const
 {
   return checkRunCommand->isChecked();
 }
@@ -877,7 +928,17 @@ const QString KBiffNewMailTab::getRunCommandPath() const
   return editRunCommand->text();
 }
 
-const bool KBiffNewMailTab::getPlaySound() const
+bool KBiffNewMailTab::getRunResetCommand() const
+{
+  return checkRunResetCommand->isChecked();
+}
+
+const QString KBiffNewMailTab::getRunResetCommandPath() const
+{
+  return editRunResetCommand->text();
+}
+
+bool KBiffNewMailTab::getPlaySound() const
 {
   return checkPlaySound->isChecked();
 }
@@ -887,17 +948,17 @@ const QString KBiffNewMailTab::getPlaySoundPath() const
   return editPlaySound->text();
 }
 
-const bool KBiffNewMailTab::getSystemBeep() const
+bool KBiffNewMailTab::getSystemBeep() const
 {
   return checkBeep->isChecked();
 }
 
-const bool KBiffNewMailTab::getNotify() const
+bool KBiffNewMailTab::getNotify() const
 {
   return checkNotify->isChecked();
 }
 
-const bool KBiffNewMailTab::getStatus() const
+bool KBiffNewMailTab::getStatus() const
 {
   return checkStatus->isChecked();
 }
@@ -908,6 +969,12 @@ void KBiffNewMailTab::enableRunCommand(bool enable)
   buttonBrowseRunCommand->setEnabled(enable);
 }
 
+void KBiffNewMailTab::enableRunResetCommand(bool enable)
+{
+  editRunResetCommand->setEnabled(enable);
+  buttonBrowseRunResetCommand->setEnabled(enable);
+}
+
 void KBiffNewMailTab::enablePlaySound(bool enable)
 {
   editPlaySound->setEnabled(enable);
@@ -932,6 +999,22 @@ void KBiffNewMailTab::browseRunCommand()
   editRunCommand->setText( url.path() );
 }
 
+void KBiffNewMailTab::browseRunResetCommand()
+{
+
+  KURL url = KFileDialog::getOpenURL();
+
+  if( url.isEmpty() )
+    return;
+
+  if( !url.isLocalFile() )
+  {
+    return;
+  }
+
+  editRunResetCommand->setText( url.path() );
+}
+
 void KBiffNewMailTab::browsePlaySound()
 {
 
@@ -949,6 +1032,7 @@ void KBiffNewMailTab::browsePlaySound()
   editPlaySound->setText( url.path() );
 }
 
+
 //////////////////////////////////////////////////////////////////////
 // KBiffMailboxTab
 //////////////////////////////////////////////////////////////////////
@@ -1008,6 +1092,13 @@ KBiffMailboxAdvanced::KBiffMailboxAdvanced()
       "If this is checked, then the socket protocols will access "
       "the server asynchronously");
   QWhatsThis::add(async, whatsthis);
+  
+  apop = new QCheckBox(i18n("&Disable APOP"), this);
+  apop->setEnabled(false);
+  whatsthis = i18n(
+      "If this is checked, then POP mailboxes will not use Authenticated POP where available, "
+      "and send passwords in plaintext over the network, which is a security risk");
+  QWhatsThis::add(apop, whatsthis);
 
   QPushButton *ok = new QPushButton(i18n("&OK"), this);
   ok->setDefault(true);
@@ -1018,6 +1109,7 @@ KBiffMailboxAdvanced::KBiffMailboxAdvanced()
   connect(preauth, SIGNAL(toggled(bool)), SLOT(preauthModified(bool)));
   connect(keepalive, SIGNAL(toggled(bool)), SLOT(keepaliveModified(bool)));
   connect(async, SIGNAL(toggled(bool)), SLOT(asyncModified(bool)));
+  connect(apop, SIGNAL(toggled(bool)), SLOT(apopModified(bool)));
   connect(port, SIGNAL(textChanged(const QString&)),
                 SLOT(portModified(const QString&)));
   connect(ok, SIGNAL(clicked()), SLOT(accept()));
@@ -1026,7 +1118,7 @@ KBiffMailboxAdvanced::KBiffMailboxAdvanced()
                    SLOT(timeoutModified(const QString&)));
 
   // NOW DO THE LAYOUT
-  QGridLayout *top_layout = new QGridLayout(this, 6, 4, 12);
+  QGridLayout *top_layout = new QGridLayout(this, 7, 4, 12);
   top_layout->addWidget(mailbox_label, 0, 0);
   top_layout->addMultiCellWidget(mailbox, 0, 0, 1, 3);
   top_layout->addWidget(port_label, 1, 0);
@@ -1036,8 +1128,9 @@ KBiffMailboxAdvanced::KBiffMailboxAdvanced()
   top_layout->addWidget(preauth, 2, 1);
   top_layout->addWidget(keepalive, 3, 1);
   top_layout->addWidget(async, 4, 1);
-  top_layout->addWidget(ok, 5, 2);
-  top_layout->addWidget(cancel, 5, 3);
+  top_layout->addWidget(apop, 4, 1);
+  top_layout->addWidget(ok, 6, 2);
+  top_layout->addWidget(cancel, 6, 3);
 }
 
 KBiffMailboxAdvanced::~KBiffMailboxAdvanced()
@@ -1051,7 +1144,7 @@ const KBiffURL KBiffMailboxAdvanced::getMailbox() const
   return url;
 }
 
-const unsigned int KBiffMailboxAdvanced::getPort() const
+unsigned int KBiffMailboxAdvanced::getPort() const
 {
   return QString(port->text()).toInt();
 }
@@ -1120,6 +1213,19 @@ void KBiffMailboxAdvanced::asyncModified(bool is_async)
   setMailbox(url);
 }
 
+void KBiffMailboxAdvanced::apopModified(bool disable_apop)
+{
+  KBiffURL url = getMailbox();
+  if ( url.protocol() == "pop3" || url.protocol() == "pop3s" )
+  {
+    if ( disable_apop )
+      url.setSearchPar("apop", "no");
+    else
+      url.setSearchPar("apop", "yes");
+    setMailbox(url);
+  }
+}
+
 void KBiffMailboxAdvanced::setPreauth(bool on)
 {
   preauth->setEnabled(true);
@@ -1138,13 +1244,19 @@ void KBiffMailboxAdvanced::setAsync(bool on)
   async->setChecked(on);
 }
 
+void KBiffMailboxAdvanced::setDisableApop(bool on)
+{
+  apop->setEnabled(true);
+  apop->setChecked(on);
+}
+
 bool KBiffMailboxAdvanced::getPreauth() const
 {
   return preauth->isChecked();
 }
 
-KBiffMailboxTab::KBiffMailboxTab(const QString& profile, QWidget *parent)
-  : QWidget(parent), mailboxHash(new QDict<KBiffMailbox>)
+KBiffMailboxTab::KBiffMailboxTab(const QString& profile_, QWidget *parent_)
+  : QWidget(parent_), mailboxHash(new QDict<KBiffMailbox>)
 {
   if (mailboxHash)
     mailboxHash->setAutoDelete(true);
@@ -1172,6 +1284,10 @@ KBiffMailboxTab::KBiffMailboxTab(const QString& profile, QWidget *parent)
   comboProtocol->insertItem("mh");
   comboProtocol->insertItem("file");
   comboProtocol->insertItem("nntp");
+#ifdef USE_SSL
+  comboProtocol->insertItem("imap4s");
+  comboProtocol->insertItem("pop3s");
+#endif // USE_SSL
   protocol_label->setBuddy(comboProtocol);
 
   QLabel *mailbox_label = new QLabel(i18n("&Mailbox:"), this);
@@ -1272,7 +1388,7 @@ KBiffMailboxTab::KBiffMailboxTab(const QString& profile, QWidget *parent)
   top_layout->addLayout(mailbox_layout);
   top_layout->addLayout(right_side_layout);
 
-  readConfig(profile);
+  readConfig(profile_);
 }
 
 KBiffMailboxTab::~KBiffMailboxTab()
@@ -1280,7 +1396,7 @@ KBiffMailboxTab::~KBiffMailboxTab()
   delete mailboxHash;
 }
 
-void KBiffMailboxTab::readConfig(const QString& profile)
+void KBiffMailboxTab::readConfig(const QString& profile_)
 {
   // initialize some variables that need initing
   oldItem = 0;
@@ -1292,7 +1408,7 @@ void KBiffMailboxTab::readConfig(const QString& profile)
   mailboxHash->clear();
   mailboxes->clear();
 
-  config->setGroup(profile);
+  config->setGroup(profile_);
 
   QStringList mailbox_list;
 
@@ -1338,13 +1454,13 @@ void KBiffMailboxTab::readConfig(const QString& profile)
   delete config;
 }
 
-void KBiffMailboxTab::saveConfig(const QString& profile)
+void KBiffMailboxTab::saveConfig(const QString& profile_)
 {
     // open the config file
     KSimpleConfig *config = new KSimpleConfig(CONFIG_FILE);
     config->setDollarExpansion(false);
 
-    config->setGroup(profile);
+    config->setGroup(profile_);
 
     QStringList mailbox_list;
 
@@ -1423,13 +1539,19 @@ void KBiffMailboxTab::setMailbox(const KBiffURL& url)
         protocolSelected(6);
     else if (prot == "nntp")
         protocolSelected(7);
+#ifdef USE_SSL
+    else if (prot == "imap4s")
+        protocolSelected(8);
+    else if (prot == "pop3s")
+        protocolSelected(9);
+#endif // USE_SSL
     else
         return;
 
     if (editMailbox->isEnabled())
     {
         QString path(url.path());
-        if (((prot == "imap4") || (prot == "nntp")) && !path.isEmpty() && path[0] == '/')
+        if (((prot == "imap4") || (prot == "nntp") || (prot == "imap4s")) && !path.isEmpty() && path[0] == '/')
             path.remove(0, 1);
 
         editMailbox->setText(path);
@@ -1454,6 +1576,7 @@ void KBiffMailboxTab::setMailbox(const KBiffURL& url)
     preauth = url.searchPar("preauth") == "yes" ? true : false;
     keepalive = url.searchPar("keepalive") == "yes" ? true : false;
     async = url.searchPar("async") == "yes" ? true : false;
+    useApop = url.searchPar("apop") == "no" ? false : true;
 
     QString fetch = url.searchPar("fetch");
     checkFetchCommand->setChecked(!fetch.isEmpty());
@@ -1492,7 +1615,7 @@ const KBiffURL KBiffMailboxTab::getMailbox() const
         url.setPath(path);
     }
 
-    if ((prot == "imap4") || (prot == "pop3") || (prot == "nntp"))
+    if ((prot == "imap4") || (prot == "pop3") || (prot == "nntp") || (prot == "imap4s") || (prot == "pop3s"))
     {
         if (keepalive)
             url.setSearchPar("keepalive", "yes");
@@ -1504,6 +1627,13 @@ const KBiffURL KBiffMailboxTab::getMailbox() const
         else
             url.setSearchPar("async", "no");
 
+        if ((prot == "pop3s") || (prot == "pop3") )
+        {
+            if (useApop)
+                url.setSearchPar("apop", "yes");
+            else
+                url.setSearchPar("apop", "no");
+        }
         url.setSearchPar("timeout", QString().setNum(timeout));
     }
 
@@ -1634,6 +1764,18 @@ void KBiffMailboxTab::protocolSelected(int protocol)
             editPassword->setEnabled(true);
             checkStorePassword->setEnabled(true);
             break;
+#ifdef USE_SSL
+        case 8: // IMAP4S
+            port = 993;
+            timeout = 10;
+            editMailbox->setEnabled(true);
+            buttonBrowse->setEnabled(true);
+            editServer->setEnabled(true);
+            editUser->setEnabled(true);
+            editPassword->setEnabled(true);
+            checkStorePassword->setEnabled(true);
+            break;
+#endif // USE_SSL
         case 7: // NNTP
             port = 119;
             timeout = 10;
@@ -1654,6 +1796,18 @@ void KBiffMailboxTab::protocolSelected(int protocol)
             editPassword->setEnabled(true);
             checkStorePassword->setEnabled(true);
             break;
+#ifdef USE_SSL
+        case 9: // POP3S
+            port = 995;
+            timeout = 10;
+            editMailbox->setEnabled(false);
+            buttonBrowse->setEnabled(false);
+            editServer->setEnabled(true);
+            editUser->setEnabled(true);
+            editPassword->setEnabled(true);
+            checkStorePassword->setEnabled(true);
+            break;
+#endif // USE_SSL
         default: // blank
             port = 0;
             timeout = 0;
@@ -1671,10 +1825,10 @@ void KBiffMailboxTab::browse()
 {
   QString proto(getMailbox().protocol());
 
-  if (proto == "imap4")
+  if (proto == "imap4" || proto == "imap4s")
   {
     KURL start;
-    start.setProtocol("imap");
+    start.setProtocol((proto == "imap4s") ? "imaps" : "imap");
     start.setUser(getMailbox().user());
     start.setHost(getMailbox().host());
     start.setPath("/");
@@ -1725,17 +1879,18 @@ void KBiffMailboxTab::advanced()
     advanced_dlg.setTimeout(timeout);
   }
 
-  if (prot == "imap4")
+  if ((prot == "imap4") || (prot == "imap4s"))
   {
     advanced_dlg.setPreauth(preauth);
     advanced_dlg.setKeepalive(keepalive);
     advanced_dlg.setAsync(async);
   }
 
-  if ((prot == "pop3") || (prot == "nntp"))
+  if ((prot == "pop3") || (prot == "nntp") || (prot == "pop3s"))
   {
     advanced_dlg.setKeepalive(keepalive);
     advanced_dlg.setAsync(async);
+    advanced_dlg.setDisableApop(!useApop);
   }
 
   advanced_dlg.setMailbox(getMailbox());
@@ -1767,8 +1922,8 @@ const KBiffURL KBiffMailboxTab::defaultMailbox() const
 //////////////////////////////////////////////////////////////////////
 // KBiffAboutTab
 //////////////////////////////////////////////////////////////////////
-KBiffAboutTab::KBiffAboutTab(QWidget *parent)
-  : QWidget(parent)
+KBiffAboutTab::KBiffAboutTab(QWidget *parent_)
+  : QWidget(parent_)
 {
   // load in the kbiff pixmap
   QPixmap logo_pixmap(kapp->icon());
@@ -1792,7 +1947,7 @@ KBiffAboutTab::KBiffAboutTab(QWidget *parent)
                       SLOT(homepage(const QString&)));
 
   QLabel *version_label = new QLabel(this);
-  version_label->setText(QString("Version %1\n\nCopyright (C) 1998-2002\nKurt Granroth").arg("3.6.3"));
+  version_label->setText(QString("Version %1\n\nCopyright (C) 1998-2008\nKurt Granroth").arg(kbiff_version));
 
   KURLLabel *email_label = new KURLLabel(this);
   email_label->setText("granroth@kde.org");
@@ -1830,8 +1985,8 @@ void KBiffAboutTab::homepage(const QString& url)
   (void) new KRun ( url );
 }
 
-KBiffNewDlg::KBiffNewDlg(QWidget* parent, const char * name)
-  : KDialog(parent, name, true)
+KBiffNewDlg::KBiffNewDlg(QWidget* parent_, const char * name_)
+  : KDialog(parent_, name_, true)
 {
   // set my name
   setCaption(i18n("New Name"));
