@@ -5,7 +5,7 @@
  * This file contains the setup dialog and related widgets
  * for KBiff.  All user configuration is done here.
  *
- * $Id: setupdlg.h,v 1.4 2001/08/03 22:52:11 gran Exp $
+ * $Id$
  */
 #ifndef SETUPDLG_H
 #define SETUPDLG_H
@@ -56,21 +56,24 @@ public:
 
 	const QString getMailClient() const;
 	const QString getRunCommandPath() const;
+	const QString getRunResetCommandPath() const;
 	const QString getPlaySoundPath() const;
 	const QString getNoMailIcon() const;
 	const QString getNewMailIcon() const;
 	const QString getOldMailIcon() const;
 	const QString getNoConnIcon() const;
-	const bool getSystemBeep() const;
-	const bool getRunCommand() const;
-	const bool getPlaySound() const;
-	const bool getNotify() const;
-	const bool getStatus() const;
-	const bool getDock() const;
-	const bool getSessionManagement() const;
-	const bool getCheckStartup() const;
-	const bool getSecure() const;
-	const unsigned int getPoll() const;
+	const QString getStoppedIcon() const;
+	bool getSystemBeep() const;
+	bool getRunCommand() const;
+	bool getRunResetCommand() const;
+	bool getPlaySound() const;
+	bool getNotify() const;
+	bool getStatus() const;
+	bool getDock() const;
+	bool getSessionManagement() const;
+	bool getCheckStartup() const;
+	bool getSecure() const;
+	unsigned int getPoll() const;
 
 public slots:
 	void invokeHelp();
@@ -111,11 +114,12 @@ public:
 	const QString getButtonOldMail() const;
 	const QString getButtonNoMail() const;
 	const QString getButtonNoConn() const;
+	const QString getButtonStopped() const;
 	const QString getMailClient() const;
-	const int   getPoll() const;
-	const bool  getDock() const;
-	const bool  getSessionManagement() const;
-	const bool  getCheckStartup() const;
+	int   getPoll() const;
+	bool  getDock() const;
+	bool  getSessionManagement() const;
+	bool  getCheckStartup() const;
 
 public slots:
 	void readConfig(const QString& profile);
@@ -132,6 +136,7 @@ private:
 	KIconButton *buttonOldMail;
 	KIconButton *buttonNewMail;
 	KIconButton *buttonNoConn;
+	KIconButton *buttonStopped;
 };
 
 class KBiffNewMailTab : public QWidget
@@ -141,13 +146,15 @@ public:
 	KBiffNewMailTab(const QString& profile = QString::null, QWidget *parent=0);
 	virtual ~KBiffNewMailTab();
 
-	const bool getRunCommand() const;
+	bool getRunCommand() const;
 	const QString getRunCommandPath() const;
-	const bool getPlaySound() const;
+	bool getRunResetCommand() const;
+	const QString getRunResetCommandPath() const;
+	bool getPlaySound() const;
 	const QString getPlaySoundPath() const;
-	const bool getSystemBeep() const;
-	const bool getNotify() const;
-	const bool getStatus() const;
+	bool getSystemBeep() const;
+	bool getNotify() const;
+	bool getStatus() const;
 
 public slots:
 	void readConfig(const QString& profile);
@@ -155,16 +162,20 @@ public slots:
 
 protected slots:
 	void enableRunCommand(bool);
+	void enableRunResetCommand(bool);
 	void enablePlaySound(bool);
 	void browseRunCommand();
+	void browseRunResetCommand();
 	void browsePlaySound();
 	void testPlaySound();
 
 private:
 	QLineEdit *editRunCommand;
+	QLineEdit *editRunResetCommand;
 	QLineEdit *editPlaySound;
 
 	QCheckBox *checkRunCommand;
+	QCheckBox *checkRunResetCommand;
 	QCheckBox *checkPlaySound;
 	QCheckBox *checkBeep;
 	QCheckBox *checkNotify;
@@ -173,6 +184,7 @@ private:
 	QPushButton *buttonBrowsePlaySound;
 	QPushButton *buttonTestPlaySound;
 	QPushButton *buttonBrowseRunCommand;
+	QPushButton *buttonBrowseRunResetCommand;
 };
 
 class KBiffMailboxAdvanced : public KDialog
@@ -183,7 +195,7 @@ public:
 	virtual ~KBiffMailboxAdvanced();
 
 	const KBiffURL getMailbox() const;
-	const unsigned int getPort() const;
+	unsigned int getPort() const;
 	bool getPreauth() const;
 
 	void setPort(unsigned int the_port, bool enable = true);
@@ -192,6 +204,7 @@ public:
 	void setPreauth(bool on);
 	void setKeepalive(bool on);
 	void setAsync(bool on);
+	void setDisableApop(bool on);
 
 protected slots:
 	void portModified(const QString& text);
@@ -199,6 +212,7 @@ protected slots:
 	void preauthModified(bool toggled);
 	void keepaliveModified(bool toggled);
 	void asyncModified(bool toggled);
+    void apopModified(bool toggled);
 
 private:
 	QString    password;
@@ -208,6 +222,7 @@ private:
 	QCheckBox *preauth;
 	QCheckBox *keepalive;
 	QCheckBox *async;
+    QCheckBox *apop;
 };
 
 class KBiffMailboxTab : public QWidget
@@ -250,6 +265,7 @@ private:
 	bool         preauth;
 	bool         keepalive;
 	bool         async;
+	bool         useApop;
 	QComboBox   *comboProtocol;
 	QLineEdit   *editMailbox;
 	QLineEdit   *editServer;
